
# q-expansion of newform 546.2.p.b, downloaded from the LMFDB on 08 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 546
weight = 2
poly_data = [1024, 0, 2112, 0, 481, 0, 38, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0, 0, 0, 0, 0], 1], [[0, 1, 0, 0, 0, 0, 0, 0], 1], [[0, 1504, 0, 223, 0, -6, 0, -1], 1024], [[-192, -800, -1094, -611, -164, -82, -6, -3], 512], [[-192, 800, -1094, 611, -164, 82, -6, 3], 512], [[704, -800, 2902, -611, 516, -82, 22, -3], 512], [[-6784, 2720, -8884, -743, -1336, -170, -52, -7], 1024], [[-7168, -1120, -11072, 1965, -1664, 334, -64, 13], 1024]]

hecke_ring_character_values = [[365, [-1, 0, 0, 0, 0, 0, 0, 0]], [157, [1, 0, 0, 0, 0, 0, 0, 0]], [379, [0, 0, -1, 0, 0, 0, 0, 0]]]
aps_data = [[0, 0, 0, 0, -1, 0, 0, 0], [0, 0, -1, -1, -1, 0, 0, 0], [1, 0, 1, 0, 0, 0, 0, 1], [0, 0, 0, 0, 1, 0, 0, 0], [1, -1, -1, -1, -1, -1, -1, 0], [-1, 1, -1, -1, 2, 0, 0, -1], [-2, 0, 0, 0, -1, -1, 0, 0], [-3, 0, 3, -1, 0, 0, -1, 0], [0, 0, 0, -1, 0, -1, -2, -2], [0, -1, 0, 0, -1, 0, -1, 1], [2, 0, -2, 2, 0, 0, 0, 0], [-3, -1, -3, 0, -1, 1, 0, 1], [3, -1, 3, 0, -1, 1, 0, 2], [0, 1, 0, 2, 0, 0, 2, -2], [0, 0, 0, -6, 0, 0, 0, 0], [0, -2, 4, 6, -7, 0, -1, 1], [1, -2, -1, 4, -2, -2, 0, 0], [0, 0, 0, 1, 4, 1, -2, -2], [3, -2, -3, -4, -2, -2, -2, 0], [-4, -1, -4, 0, -1, 1, 0, 0], [3, 1, 3, 0, 9, -1, 0, 1], [2, 0, 0, 1, 2, 0, -1, -1], [3, -1, 3, 0, 1, 1, 0, 0], [-1, 2, 1, 6, 2, 2, 0, 0], [4, 0, -4, -2, 0, 0, 0, 0], [0, 0, 0, -5, -4, 2, 1, 1], [0, 3, 0, 4, -1, 0, 3, -3], [0, 0, 2, -2, 2, 0, 0, 0], [-1, -3, 1, 1, -3, -3, -5, 0], [0, 0, 4, -6, 6, 0, 0, 0], [0, 2, 6, -9, 8, 0, -1, 1], [0, 1, 4, 3, -4, 0, -1, 1], [7, 0, -7, 5, 0, 0, 3, 0], [12, 0, 0, 2, 0, -2, 0, 0], [-4, 1, -4, 0, 3, -1, 0, -6], [1, 0, 1, 0, 8, 0, 0, 1], [0, 0, 0, -1, 0, 5, 4, 4], [1, 0, 1, 0, 2, 0, 0, 4], [3, -1, -3, 7, -1, -1, 1, 0], [-2, 0, 0, -10, -12, 0, 2, 2], [8, 0, 0, 1, 0, -2, -1, -1], [0, -2, 10, 2, -3, 0, -1, 1], [0, 1, 4, -5, 3, 0, -2, 2], [-5, -1, -5, 0, -3, 1, 0, -4], [-2, -3, -2, 0, -1, 3, 0, 0], [0, -5, -12, -2, -1, 0, -3, 3], [-10, 0, 0, 4, 5, -1, -2, -2], [-2, 5, 2, 2, 5, 5, 2, 0], [-3, 0, -3, 0, 0, 0, 0, 4], [2, 0, 2, 0, -8, 0, 0, 2], [-14, 0, 0, 1, 0, -2, -1, -1], [-4, 2, -4, 0, 2, -2, 0, 0], [4, 0, 4, 0, -14, 0, 0, 2], [8, 0, 0, -9, -9, -1, -1, -1], [4, 0, 0, -4, -4, 2, 2, 2], [0, 4, 10, 0, 3, 0, 3, -3], [0, 2, 0, -1, 2, 0, 1, -1], [-4, -1, -4, 0, 1, 1, 0, 0], [0, 6, 12, -3, 4, 0, 1, -1], [4, 2, -4, -4, 2, 2, 6, 0], [0, -6, 10, -3, 4, 0, 1, -1], [2, -4, -2, 2, -4, -4, -6, 0], [10, -2, 10, 0, -14, 2, 0, 6], [10, 0, 0, -1, 0, 4, 3, 3], [4, 0, 0, -5, -4, -4, -5, -5], [12, 2, 12, 0, 8, -2, 0, 0], [-3, -2, 3, -2, -2, -2, -4, 0], [0, 3, 14, -3, 4, 0, 1, -1], [0, -2, 2, 4, -4, 0, 0, 0], [-8, 1, -8, 0, -5, -1, 0, -4], [-3, 2, -3, 0, -8, -2, 0, -4], [-4, -1, 4, -14, -1, -1, -4, 0], [8, 0, 0, 6, 1, 0, 5, 5], [0, 0, 0, -3, -4, 0, 1, 1], [7, 1, -7, 12, 1, 1, -2, 0], [-5, 3, -5, 0, 3, -3, 0, -5], [-8, 0, 0, -6, -11, -4, 1, 1], [-10, 3, -10, 0, 7, -3, 0, -2], [14, 0, -14, -2, 0, 0, -4, 0], [-13, 3, 13, 13, 3, 3, 3, 0], [0, 1, -2, -7, 8, 0, 1, -1], [6, 2, -6, 4, 2, 2, 10, 0], [8, 0, 8, 0, -22, 0, 0, 0], [0, 0, 14, 0, 0, 0, 0, 0], [0, 1, 28, -8, 5, 0, -3, 3], [0, 0, -6, -2, 6, 0, 4, -4], [-9, 6, 9, -3, 6, 6, 3, 0], [1, -1, -1, 14, -1, -1, 2, 0], [13, 2, 13, 0, 14, -2, 0, 1], [-5, 0, -5, 0, 0, 0, 0, -1], [6, 0, 0, 7, 3, -5, -1, -1], [-5, 5, 5, 13, 5, 5, 11, 0], [16, 0, -16, 2, 0, 0, 2, 0], [-16, 0, 0, 5, 6, 2, 1, 1], [-11, -4, 11, 4, -4, -4, -2, 0], [0, 0, -8, 6, -6, 0, 0, 0], [9, -2, 9, 0, -2, 2, 0, -1], [0, 3, 22, 0, 0, 0, 0, 0], [0, 0, 0, 16, 16, -2, -2, -2], [-7, 1, -7, 0, -7, -1, 0, -9], [-18, 0, 0, -2, 0, -2, -4, -4], [-10, -3, 10, 0, -3, -3, 2, 0], [-30, 0, 0, -1, 3, 3, -1, -1], [16, 0, 0, 2, 2, 6, 6, 6], [0, -4, 10, 4, -6, 0, -2, 2], [-12, -2, 12, -8, -2, -2, 2, 0], [-9, -2, -9, 0, 14, 2, 0, 4], [-1, -6, 1, -8, -6, -6, -6, 0], [0, 5, -4, -3, 5, 0, 2, -2], [-2, 0, 0, 14, 10, -4, 0, 0], [8, 0, 0, 2, 2, 7, 7, 7], [-13, -3, 13, -9, -3, -3, 1, 0], [-3, -6, -3, 0, 12, 6, 0, 7], [9, 0, 9, 0, 8, 0, 0, -3], [-11, -6, -11, 0, -2, 6, 0, 7], [2, 0, 0, -9, -6, 8, 5, 5], [13, 2, -13, 5, 2, 2, 5, 0], [-14, 0, 0, 3, 6, 4, 1, 1], [0, -3, -20, -2, -3, 0, -5, 5], [0, -2, 6, -8, 10, 0, 2, -2], [-2, 2, -2, 0, 28, -2, 0, -2], [0, 3, -10, -1, -4, 0, -5, 5], [0, 2, -4, -9, 6, 0, -3, 3], [7, -5, -7, -13, -5, -5, -9, 0], [8, -6, -8, 8, -6, -6, 4, 0], [-20, 0, 0, 0, 7, 6, -1, -1], [10, -6, 10, 0, 4, 6, 0, 0], [-18, 0, 0, 11, 6, -12, -7, -7], [0, 3, -24, 2, -1, 0, 1, -1], [-8, -4, 8, -8, -4, -4, 4, 0], [-7, -6, -7, 0, -4, 6, 0, 0], [12, 1, 12, 0, 15, -1, 0, -4], [0, 0, 12, -6, 8, 0, 2, -2], [-2, 0, 0, -14, -14, -2, -2, -2], [9, -2, -9, 12, -2, -2, -6, 0], [-7, -3, 7, 17, -3, -3, -5, 0], [-3, 2, -3, 0, -30, -2, 0, -3], [-3, -8, -3, 0, -4, 8, 0, 5], [0, 0, 0, -5, -4, 0, -1, -1], [0, 4, 2, -15, 12, 0, -3, 3], [3, -8, -3, -7, -8, -8, -3, 0], [18, 4, 18, 0, -6, -4, 0, -2], [0, -2, 12, -14, 6, 0, -8, 8], [3, -7, -3, 21, -7, -7, 1, 0], [0, 5, 16, 19, -17, 0, 2, -2], [12, 4, -12, -24, 4, 4, 6, 0], [-6, -4, -6, 0, -6, 4, 0, 10], [18, 0, 0, 18, 8, -8, 2, 2], [-26, 0, 0, 1, 8, 8, 1, 1], [10, 3, 10, 0, 11, -3, 0, 2], [-20, 6, 20, 8, 6, 6, 6, 0], [-8, 0, 0, -22, -17, 1, -4, -4], [0, 1, 18, -12, 10, 0, -2, 2], [0, -2, 2, 21, -18, 0, 3, -3], [0, 0, -8, -14, 12, 0, -2, 2], [0, 5, -8, -3, 9, 0, 6, -6], [0, 0, 0, 6, 12, 10, 4, 4], [-11, -6, -11, 0, -4, 6, 0, -4], [8, 0, 0, -5, -8, -12, -9, -9], [-8, -2, -8, 0, -10, 2, 0, 2], [1, 7, -1, -7, 7, 7, 5, 0], [-2, 0, 0, 7, -2, -14, -5, -5], [-1, -2, 1, 13, -2, -2, -3, 0], [0, -6, -4, -2, 4, 0, 2, -2], [-11, 2, -11, 0, 36, -2, 0, -5], [-19, 7, 19, -7, 7, 7, 3, 0], [18, 0, 0, -13, -14, -4, -3, -3], [-14, 0, 0, 6, 5, 6, 7, 7]]
